from datetime import date

from django.db import models

from .settings import ACCOUNTS


class Volunteer(models.Model):
    realname = models.CharField(max_length=200, null=True)
    email = models.CharField(max_length=200, null=True)
    # username = models.CharField(max_length=200, null=True)

    # the following Fields are not supposed to be edited by users
    granted = models.BooleanField(null=True)
    granted_date = models.DateField(null=True)
    survey_mail_send = models.BooleanField(null=True)

    @classmethod
    def set_granted(cl, key, b):
        obj = cl.objects.get(pk=key)
        obj.granted = b
        obj.granted_date = date.today()
        obj.save()

    class Meta:
        abstract = True


class Extern(Volunteer):
    ''' abstract basis class for all data entered by extern volunteers '''
    username = models.CharField(max_length=200, null=True)

    # the following Fields are not supposed to be edited by users
    service_id = models.CharField(max_length=15, null=True, blank=True)

    def save(self,*args,**kwargs):
        # is there a way to call super().save() only once?
        super().save(*args,*kwargs)
        self.service_id = type(self).__name__ + str(self.pk)
        super().save(*args,*kwargs)

    class Meta:
        abstract = True

class Project(Volunteer):
    name = models.CharField(max_length=200)
    start = models.DateField('Startdatum', null=True)
    end = models.DateField('Erwartetes Projektende', null=True)
    account = models.CharField('Kostenstelle', max_length=5,
                                choices=ACCOUNTS.items(), null=True,)

    # the following Fields are not supposed to be edited by users
    pid = models.CharField(max_length=15, null=True, blank=True)
    end_mail_send = models.BooleanField(null=True)
    status = models.CharField(max_length=3,choices=(('RUN', 'läuft'),('END','beendet')),default='RUN')
    persons = models.IntegerField(default=1)

    def save(self,*args,**kwargs):
        # is there a way to call super().save() only once?
        super().save(*args,*kwargs)
        self.pid = self.account + str(self.pk)
        super().save(*args,*kwargs)

    def __str__(self):
        return f"{self.pid} {self.name}"


class Intern(Volunteer):
    '''abstrat base class for data entry from /intern (except Project)'''
    request_url = models.CharField(max_length=2000)

    class Meta:
        abstract = True


class HonoraryCertificate(Intern):
    ''' this class is also used for accreditations '''

    # request_url = models.CharField(max_length=2000)
    project = models.ForeignKey(Project, null=True, blank=True, on_delete=models.SET_NULL)

    def __str__(self):
        return "Certificate for " + self.realname


class Travel(Intern):
    project = models.ForeignKey(Project, on_delete=models.CASCADE)



#abstract base class for Library and IFG
class Grant(Extern):
    cost = models.CharField(max_length=10)
    notes = models.CharField(max_length=500, blank=True)

    class Meta:
        abstract = True


TYPE_CHOICES = {'BIB': 'Bibliotheksstipendium',
           'ELIT': 'eLiteraturstipendium',
           'SOFT': 'Softwarestipendium',
           'VIS': 'Visitenkarten',
           'LIST': 'Mailingliste',
           'MAIL': 'E-Mail-Adresse',
           'IFG': 'Kostenübernahme IFG-Anfrage',
           'LIT': 'Literaturstipendium',}


# same model is used for Library, ELitStip and Software!
class Library(Grant):

    type = models.CharField(
        max_length=4,
        choices=TYPE_CHOICES.items(), #attention: actually only BIB, ELIT, SOFT should be used here
        default='LIB',
    )
    library = models.CharField(max_length=200)
    duration = models.CharField(max_length=100)

    def __str__(self):
        return self.library

class Literature(Grant):
    info = models.CharField(max_length=500)
    source = models.CharField(max_length=200)

class IFG(Grant):
    url = models.CharField(max_length=2000)

    def __str__(self):
        return "IFG-Anfrage von " + self.realname

DOMAIN_CHOICES = {'PEDIA': '@wikipedia.de',
                  'BOOKS': '@wikibooks.de',
                  'QUOTE': '@wikiquote.de',
                  'SOURCE': '@wikisource.de',
                  'VERSITY': '@wikiversity.de',}

class Domain(Extern):
    domain = models.CharField(max_length=10,
                              choices=DOMAIN_CHOICES.items(),
                              default='PEDIA')

    class Meta:
        abstract = True

MAIL_CHOICES = {'REALNAME': 'Vorname.Nachname',
                'USERNAME': 'Username',
                'OTHER': 'Sonstiges:'}

class Email(Domain):
    address = models.CharField(max_length=50,
                              choices=MAIL_CHOICES.items(),
                              default='USERNAME', verbose_name='Adressbestandteil')
    other = models.CharField(max_length=50,blank=True,null=True)

class List(Domain):
    address = models.CharField(max_length=50, default='NO_ADDRESS')

PROJECT_CHOICE = {'PEDIA': 'Wikipedia',
                  'SOURCE': 'Wikisource',
                  'BOOKS': 'Wikibooks',
                  'QUOTE': 'Wikiquote',
                  'VERSITY': 'Wikiversity',
                  'VOYAGE': 'Wikivoyage',
                  'DATA': 'Wikidata',
                  'NEWS': 'Wikinews',
                  'COMMONS': 'Wikimedia Commons'}

BC_VARIANT = {'PIC': 'Mit Bild',
              'NOPIC': 'Ohne Bild'}

class BusinessCard(Extern):
    project = models.CharField(max_length=20, choices=PROJECT_CHOICE.items(),
                               default='PEDIA', verbose_name='Wikimedia Projekt')

    data = models.CharField(max_length=1000, verbose_name='Persönliche Daten', default='')
    variant = models.CharField(max_length=5, choices=BC_VARIANT.items(),
                               default='NOPIC', verbose_name='Variante')
    sent_to = models.CharField(max_length=1000, verbose_name='Sendungsadresse',
                               default='')
